<?php

namespace FiloBlu\Flow\Model\Channel\In;

use Magento\ImportExport\Model\Import\Adapter as ImportAdapter;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\ImportExport\Model\Import;

class Images extends AbstractModel {
    
        /**
    * @var Monolog\Logger
    */
    protected $_logger;

    private $channel_photo_config;
    private $collection_of_images_to_parse;
    private $array_of_data_for_csv;
    private $filename_separator = "_";
    private $image_alt_separator = " ";

    public function __construct(
             \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
             \FiloBlu\Flow\Model\Channel\Config $channelConfig
    ) {
        parent::__construct($loggerProvider, $channelConfig);
        $this->_logger = $loggerProvider->getLogger();
    }

    public function load($channelname) {
        $this->init($channelname);
    }       
    
    public function insertData($file){
        $filename_to_import = $file->getName();
        $meta_file_id = $file->getId();
        
        $this->load($file->getChannel());
        $this->loadImageChannelConfig($file->getChannel());        
        $channel_config = $this->_connector->getConfig();
        $channel_config_local_dir = $channel_config['local_dir'];
        
        $local_dir = $this->_connector->getLocalDir()."/".$meta_file_id;          
        $this->_connector->setLocalDir($local_dir);
        
        $sourceFileDir = "vars/".$channel_config_local_dir."/".$meta_file_id;
        
        //$sourceFileDir = $local_dir;

        $modelImport = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\ImportExport\Model\Import');        

        $data = array(
            'form_key' => '', //This can be empty? fVzmgkRU4lwm2m1D
            'entity' => 'catalog_product',
            'behavior' => 'append',
            'validation_strategy' => 'validation-stop-on-errors',
            'allowed_error_count' => '10',
            '_import_field_separator' => ',',
            '_import_multiple_value_separator' => ',',
            'import_images_file_dir' => $sourceFileDir,
        );

        /** @var $import \Magento\ImportExport\Model\Import */
        $modelImport->setData($data);               
        
        $source = ImportAdapter::findAdapterFor(
                    $sourceFileDir."/".$filename_to_import,
                    \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Framework\Filesystem')->getDirectoryWrite(DirectoryList::ROOT),
                    ","
                );

        $result = $modelImport->validateSource($source);
        $result = $modelImport->importSource();
        
        return $result;
    }

    public function receiveFile($file){
        
        if (!$this->_connector)
            throw new \Exception('no connector joined to channel');
        
        $meta_file_id = $file->getId();       
        
        $local_dir = $this->_connector->getLocalDir()."/".$meta_file_id;      
        
        $this->_connector->setLocalDir($local_dir);
        
        $model = \Magento\Framework\App\ObjectManager::getInstance()->create('FiloBlu\Flow\Model\Imagesflow');
        
        $collection = $model->getCollection();
        $collection->addFieldToFilter("meta_processed",0);
        $collection->addFieldToFilter("meta_file",$meta_file_id);
        $collection->addFieldToFilter("image_status","found");
        $collection->load();
        
        foreach ($collection as $image) {
            try{
                $received = $this->_connector->receiveResource($image->getData('image_name'),null,FTP_BINARY,true);                
                if ($received) {
                    $image->setData('image_receive_time',date("Y-m-d H:i:s"));
                    $image->setData('image_status','received');
                    $image->save();
                }                
            }catch (\Exception $e){
                $log = $image->getData('log');
                $log = $log."\n".$e->getMessage();
                $image->setData('log',$log);
                $image->save();
                //$this->_logger->log('FTP',$e->getMessage());
            }
        }         
        $this->_connector->close();
        
        return true;        

    }
    
    
    public function parse($file){              
        if (!$this->_connector)
            throw new \Exception('no connector joined to channel');
        
        $meta_file_id = $file->getId();
        
        $local_dir = $this->_connector->getLocalDir()."/".$meta_file_id;                                
        $this->_connector->setLocalDir($local_dir);    
        
        $csv_filename = $file->getName();
        $channel_name = $file->getChannel();
        $this->loadImageChannelConfig($channel_name);
        $image_name_separator = "_";

        if (!$this->channel_photo_config) {
            //Handle error here
        }
        
        $this->loadImagesListToParse($meta_file_id);

        foreach ($this->collection_of_images_to_parse as $data){
            $image_filename = $data->getImageName();
            $loaded = $this->loadDataBasedOnFilename($image_filename);
            if ($loaded) {
                $data->setData('meta_processed',1);
                $data->save();
            }
        }
        
        $writed = $this->writeCsv($csv_filename);
        
        
        die($this->_connector->getLocalDir());
    }
    
    private function loadImageChannelConfig($channel_name){
        $this->channel_photo_config = null;
        if ($this->_parser->loadMap($channel_name)) {
            $config_mapping = $this->_parser->map->getFullMappedData();
            $this->channel_photo_config = $config_mapping;
        }
    }
    
    private function loadImagesListToParse($file_id){
        $model = \Magento\Framework\App\ObjectManager::getInstance()->create('FiloBlu\Flow\Model\Imagesflow');

        $collection = $model->getCollection();
        $collection->addFieldToFilter("meta_processed",0);
        $collection->addFieldToFilter("meta_file",$file_id);
        $collection->addFieldToFilter("image_status","received");
        $collection->load();                
        
        $this->collection_of_images_to_parse = $collection;
        
    }
    
    private function loadDataBasedOnFilename($image_filename){
        $original_filename = $image_filename;
        
        // Remove extension
        $p = strrpos($image_filename, '.');
        if ($p !== false){
            $image_filename = substr($image_filename, 0, $p);
        }
                
        $filename_exploded = explode($this->filename_separator, $image_filename);
        
        $sku_position = $this->getSkuFilenamePosition();
        $sku = $filename_exploded[$sku_position];
        
        $image_position_position = $this->GetImagePositionPosition();
        $position = $filename_exploded[$image_position_position];
        
        $image_alt_positions = $this->GetImageAltPositions();
        $alt = array();
        if (count($image_alt_positions) > 0) {
            foreach ($image_alt_positions as $pos) {
                $alt[] = $filename_exploded[$pos];
            }
            $alt = implode($this->image_alt_separator, $alt);
        } else {
            $alt = "";
        }        

        $image_type = $this->getImageType($position);

        $this->array_of_data_for_csv[$sku][$position]['filename'] = $original_filename;
        $this->array_of_data_for_csv[$sku][$position]['path'] = $this->_connector->getLocalDir();
        $this->array_of_data_for_csv[$sku][$position]['position'] = $position;
        $this->array_of_data_for_csv[$sku][$position]['alt'] = $alt;
        $this->array_of_data_for_csv[$sku][$position]['image_type'] = $image_type;
        $this->array_of_data_for_csv[$sku][$position]['sku'] = $sku; 
        
        return true;
    }
    
    
    private function getSkuFilenamePosition(){
        $config = $this->channel_photo_config;
        foreach ($config['fields'] as $key => $value) {
            if ($value['type'] == 'sku') {
                return $key;
            }            
        }
        
        //Handle error missing SKU on config here, block all flow
    }
    
    private function GetImagePositionPosition(){
        $config = $this->channel_photo_config;
        foreach ($config['fields'] as $key => $value) {
            if ($value['type'] == 'position') {
                return $key;
            }            
        }
        
        //Handle error missing SKU on config here, block all flow
    }    
    
    private function GetImageAltPositions(){
        $config = $this->channel_photo_config;
        $positions = array();
        foreach ($config['fields'] as $key => $value) {
            if ($value['type'] == 'alt') {
                $positions[] = $key;
            }            
        }
        
        return $positions;
        
        //Handle error missing SKU on config here, block all flow
    }
    
    private function getImageType($position){
        $config = $this->channel_photo_config;
        
        if (isset($config['types'][$position]['type'])) {
            return $config['types'][$position]['type'];
        }
        
        return array("additional");

    }
    
    private function writeCsv($csv_filename) {
        $csv_rows = array();

        foreach ($this->array_of_data_for_csv as $sku => $sku_data) {
            $row = array();
            $row['sku'] = $sku;
            $row['additional_images'] = "";
            $row['additional_image_labels'] = "";
            foreach ($sku_data as $data) {
                if ($this->isBaseImage($data['image_type'])) {
                    $row['base_image'] = $data['filename'];
                    $row['base_image_label'] = $data['alt'];
                }
                if ($this->isSmallImage($data['image_type'])) {
                    $row['small_image'] = $data['filename'];
                    $row['small_image_label'] = $data['alt'];
                }
                if ($this->isThumbnailImage($data['image_type'])) {
                    $row['thumbnail_image'] = $data['filename'];
                    $row['thumbnail_image_label'] = $data['alt'];
                }
                if ($this->isAdditionalImage($data['image_type'])) {
                    $row['additional_images'] .= $data['filename'] . ",";
                    $row['additional_image_labels'] .= $data['alt'] . ",";
                }
            }
            $row['additional_images'] = rtrim($row['additional_images'], ",");
            $row['additional_image_labels'] = rtrim($row['additional_image_labels'], ",");

            $csv_rows[] = $row;
        }


        $fp1 = fopen($this->_connector->getLocalDir().DIRECTORY_SEPARATOR.$csv_filename, 'w');
        fputcsv($fp1, array_keys($csv_rows[0]));
        foreach ($csv_rows as $row) {
            fputcsv($fp1, $row);
        }
        fclose($fp1);
        
        return true; //add check if file exists
    }

    private function isBaseImage($image_types_array) {
        
        if (count($image_types_array) > 0) {
            foreach ($image_types_array as $key => $value) {
                if ($value == 'base') {
                    return true;
                }
            }
        }
        return false;
    }

    private function isThumbnailImage($image_types_array) {
        if (count($image_types_array) > 0) {
            foreach ($image_types_array as $key => $value) {
                if ($value == 'thumbnail') {
                    return true;
                }
            }
        }
        return false;
    }

    private function isSmallImage($image_types_array) {

        if (count($image_types_array) > 0) {
            foreach ($image_types_array as $key => $value) {
                if ($value == 'small') {
                    return true;
                }
            }
        }
        return false;
    }

    private function isAdditionalImage($image_types_array) {
        if (count($image_types_array) > 0) {
            foreach ($image_types_array as $key => $value) {
                if ($value == 'additional') {
                    return true;
                }
            }
        }
        return false;
    }

}

